"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var rxjs_audio_utils_1 = require("./rxjs-audio-utils");
var AudioStream = /** @class */ (function () {
    function AudioStream(input, config) {
        this.audio = new Audio();
        this.currentTrack = 0;
        this.audioInput = [];
        this.audioEvents = [
            'ended',
            'error',
            'play',
            'playing',
            'pause',
            'timeupdate',
            'canplay',
            'loadedmetadata',
            'loadstart'
        ];
        this.state = {
            playing: false,
            isFirstTrack: false,
            isLastTrack: false,
            trackInfo: {
                currentTrack: undefined,
                readableCurrentTime: '',
                readableDuration: '',
                duration: undefined,
                currentTime: undefined
            }
        };
        this.stateChange = new rxjs_1.Subject();
        this.config = config;
        this.configureStream(this.config, input);
    }
    /**
     * Play the Audio Stream
     */
    AudioStream.prototype.play = function () {
        this.audio.play();
    };
    /**
     * Pause the Audio Stream
     */
    AudioStream.prototype.pause = function () {
        this.audio.pause();
    };
    /**
     * Stop the Audio Stream
     */
    AudioStream.prototype.stop = function () {
        this.audio.currentTime = 0;
        this.audio.pause();
    };
    /**
     * Select the Next Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    AudioStream.prototype.next = function () {
        if (!this.isLastPlaying()) {
            this.initAudio(++this.currentTrack);
        }
    };
    /**
     * Select the Previous Audio Item in Playlist.
     * But it doens't play it automatically.
     * Run `play()` to play the stream
     */
    AudioStream.prototype.previous = function () {
        if (!this.isFirstPlaying()) {
            this.initAudio(--this.currentTrack);
        }
    };
    /**
     * Method to Check if First Track is Playing or not
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isFirstTrack:boolean` to check if First Track is playing or not
     */
    AudioStream.prototype.isFirstPlaying = function () {
        if (this.currentTrack === 0) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * Method to Check if Last Track is Playing or not.
     * Alternatively you can `this.state()` Observable to get state.
     * state has `isLastTrack:boolean` to check if First Track is playing or not
     */
    AudioStream.prototype.isLastPlaying = function () {
        if (this.currentTrack === this.audioInput.length - 1) {
            return true;
        }
        else {
            return false;
        }
    };
    /**
     * Method to play track at particular index
     */
    AudioStream.prototype.playAt = function (index) {
        if (index < this.audioInput.length) {
            this.initAudio(index);
            this.play();
        }
    };
    /**
     * Switch the currrent Track to particular index
     * It just switch the track, doesn't play the track.
     * To do switch and play, use `platAt()` method.
     */
    AudioStream.prototype.switchTo = function (index) {
        if (index < this.audioInput.length) {
            this.initAudio(index);
        }
    };
    /**
     * Seek audio to particular time
     *
     * @param time
     */
    AudioStream.prototype.seekTo = function (time) {
        this.audio.currentTime = time;
    };
    /**
     * Returns Observable which listens to audio events of audio stream
     */
    AudioStream.prototype.events = function () {
        var _this = this;
        return rxjs_1.Observable.create(function (observer) {
            var handler = function (event) {
                observer.next(event);
                _this.updateStateEvents(event);
            };
            _this.addEvents(_this.audio, _this.audioEvents, handler);
            return function () {
                _this.removeEvents(_this.audio, _this.audioEvents, handler);
            };
        }).pipe(operators_1.share());
    };
    /**
     * Method to get stream of state updates
     */
    AudioStream.prototype.getState = function () {
        return this.stateChange;
    };
    /**
     * initialize the audio stream
     * internal implementation, don't use it.
     */
    AudioStream.prototype.initAudio = function (index) {
        /** reset the current audio */
        if (this.audio) {
            this.stop();
        }
        this.currentTrack = index;
        this.audio.src = this.audioInput[this.currentTrack];
        this.audio.load();
        this.updateStateProps(this.state.trackInfo, 'currentTrack', this.currentTrack);
        this.updateStateProps(this.state, 'isFirstTrack', this.isFirstPlaying());
        this.updateStateProps(this.state, 'isLastTrack', this.isLastPlaying());
    };
    /**
     * method to add events listener for audio stream
     * internal implementation, don't use it.
     */
    AudioStream.prototype.addEvents = function (obj, events, handler) {
        events.forEach(function (event) {
            obj.addEventListener(event, handler);
        });
    };
    /**
     * method to remove events listeners for audio stream
     * internal implementation, don't use it.
     */
    AudioStream.prototype.removeEvents = function (obj, events, handler) {
        events.forEach(function (event) {
            obj.removeEventListener(event, handler);
        });
    };
    /**
     * method to update state from audio events
     * internal implementation, don't use it.
     */
    AudioStream.prototype.updateStateEvents = function (event) {
        var trackInfo = this.state.trackInfo;
        switch (event.type) {
            case 'canplay':
                trackInfo.duration = this.audio.duration;
                trackInfo.readableDuration = rxjs_audio_utils_1.formatTime(trackInfo.duration);
                this.state.playing = false;
                break;
            case 'playing':
                this.state.playing = true;
                break;
            case 'pause':
                this.state.playing = false;
                break;
            case 'timeupdate':
                trackInfo.currentTime = this.audio.currentTime;
                trackInfo.readableCurrentTime = rxjs_audio_utils_1.formatTime(trackInfo.currentTime);
                break;
            case 'error':
                this.state.playing = false;
                trackInfo.currentTime = 0;
                trackInfo.duration = 0;
                trackInfo.readableDuration = rxjs_audio_utils_1.formatTime(trackInfo.duration);
                trackInfo.readableCurrentTime = rxjs_audio_utils_1.formatTime(trackInfo.currentTime);
                break;
        }
        this.stateChange.next(this.state);
    };
    /**
     * method to update state from properties
     * internal implementation, don't use it.
     */
    AudioStream.prototype.updateStateProps = function (obj, prop, value) {
        obj[prop] = value;
        this.stateChange.next();
    };
    /**
     * extract tracks url from Array of tracks
     * internal implementation, don't use it.
     */
    AudioStream.prototype.extractTracks = function (tracks, key) {
        return tracks.map(function (track) {
            return track[key];
        });
    };
    /**
     * autoplay the next track if set in config
     * internal implementation, don't use it.
     */
    AudioStream.prototype.autoPlayNext = function () {
        if (this.config && this.config.autoPlayNext) {
            this.next();
            this.play();
        }
    };
    /**
     * config the stream for initialization
     * internal implementation, don't use it.
     */
    AudioStream.prototype.configureStream = function (config, input) {
        var _this = this;
        var initialTrack;
        /** if there is config available */
        /*  extract tracks from a list of track */
        if (config && config.urlKey) {
            this.audioInput = this.extractTracks(input, config.urlKey);
        }
        else {
            this.audioInput = input;
        }
        /* set initial track based on user input */
        if (config && config.initialTrack) {
            initialTrack = config.initialTrack;
        }
        else {
            initialTrack = 0;
        }
        /** if there is no config, do the following  */
        if (!config) {
            initialTrack = 0;
            this.audioInput = input;
        }
        /* initialize audio **/
        this.initAudio(initialTrack);
        /** if autoPlayNext is set to true */
        var canPlay$ = this.events().pipe(operators_1.filter(function (event) { return event.type === 'canplay'; }));
        var ended$ = this.events().pipe(operators_1.filter(function (event) { return event.type === 'ended'; }));
        canPlay$.pipe(operators_1.switchMap(function (event) { return ended$; })).subscribe(function (value) { return _this.autoPlayNext(); });
    };
    return AudioStream;
}());
exports.AudioStream = AudioStream;
//# sourceMappingURL=rxjs-audio-stream.js.map